#include <iostream.h>
#include <stdlib.h>
#include <string.h>
#include <conio.h>
#include <math.h>

//CLASSE STRINGA

class String
{
 private:
	char *itsString;
	unsigned short itsLen;
	String (unsigned short); //costruttore privato
 public:
	//Costruttori
	String();
	String(const char *const);
	String(const String &);
	~String();
	//Operatori overloaded
	char & operator[](unsigned short offset);
	char operator[](unsigned short offset)const;
	String operator+(const String &);
	void operator+=(const String &);
	String & operator=(const String &);
	int operator==(const String &);
	//Metodi di accesso generali
	unsigned short GetLen() const {return itsLen;}
	const char* GetString() const {return itsString;}
	void Leggi();
};

//Costruttore di default crea una stringa con 0 byte
String::String()
{	itsString=new char[1];
	itsString[0]='\0';
	itsLen=0;
}

//Costruttore privato, usato solo dai metodi della classe per
//creare una nuova stringa della dimensione richiesta
//inizializzata dal carattere null.
String::String(unsigned short len)
{	itsString=new char[len+1];
	for(unsigned short i=0; i<=len; i++)
		itsString[i]='\0';
	itsLen=len;
}

//Costruttore che converte a stringa un array di caratteri,
//il parametro  un puntatore costante ad oggetto costante
String::String(const char *const cString)
{  itsLen=strlen(cString);
	itsString=new char[itsLen+1];
	for(unsigned short i=0; i<=itsLen; i++)
		itsString[i]=cString[i];
	itsString[itsLen]='\0';
}

//Costruttore di copia
String::String(const String &rhs)
{   itsLen=rhs.GetLen();
	itsString=new char[itsLen+1];
	for(unsigned short i=0; i<=itsLen; i++)
		itsString[i]=rhs[i];
	itsString[itsLen]='\0';
}

//Distruttore
String::~String()
{  delete [] itsString;
	itsLen=0;
}

//Operatore = libera la memoria esistente e poi
//copia la stringa e le dimensioni
String & String::operator=(const String &rhs)
{  if (this==&rhs)
	return *this;
	delete [] itsString;
	itsLen=rhs.GetLen();
	itsString=new char[itsLen+1];
	for(unsigned short i=0; i<=itsLen; i++)
		itsString[i]=rhs[i];
	itsString[itsLen]='\0';
	return *this;
}


//Operatore ==
int String::operator==(const String &rhs)
{  return (!strcmp(itsString,rhs.GetString()));
}

//Operatore [] non costante, restituisce un reference
//al carattere indicizzato per poterlo cambiare,
//usato come target di una assegnazione
char & String::operator[](unsigned short offset)
{	if(offset>itsLen)
		//Se si esce dalle dimensioni si restituisce l'ultimo char
		return itsString[itsLen-1];
	else
		//Si restituisce l'elemento di indice offset
		return itsString[offset];
}

//Operatore [] costante, da usare con oggetti costanti
//per es. col costruttore copia o nelle espressioni
//dove l'operatore pu restituire un non reference o un reference const.
char String::operator[](unsigned short offset) const
{	if(offset>itsLen)
		//Se si esce dalle dimensioni si restituisce l'ultimo char
		return itsString[itsLen-1];
	else
		//Si restituisce l'elemento di indice offset
		return itsString[offset];
}

//Crea una nuova stringa concatenando alla
//stringa corrente la stringa rhs.
String String::operator+(const String &rhs)
{	unsigned short totLen=itsLen+rhs.GetLen();
	String temp(totLen);	//si chiama il costruttore privato
	for (unsigned short i=0; i<itsLen; i++)
		temp[i]=itsString[i];
	for (unsigned short j=0; j<rhs.GetLen(); j++, i++)
		temp[i]=rhs[j];
	temp[totLen]='\0';
	return temp;
}

//Cambia la stringa corrente, non restituisce nulla,
//concatenando alla stringa corrente rhs in coda.
void String::operator+=(const String &rhs)
{	unsigned short rhsLen=rhs.GetLen();
	unsigned short totLen=itsLen+rhs.GetLen();
	String temp(totLen);
	for (unsigned short i=0; i<itsLen;i++)
		temp[i]=itsString[i];
	for (unsigned short j=0; j<rhsLen; j++,i++)
		temp[i]=rhs[i-itsLen];	//Si copia rhs
	temp[totLen]='\0';
	*this=temp;
}

void String::Leggi()
{ char temp[30];
  cin>>temp;
  *this=temp;
}


//CLASSE ITERATORE

template <class T>
class Iterator
{
	protected:
		// Indica se  finita la lista
		// deve essere aggiornata dalla classe derivata
		int iterationComplete;

	public:
		// costruttore
		Iterator();

		// Metodi richiesti per l'iteratore
		virtual void Next() = 0;
		virtual void Reset() = 0;

		// Metodo per recupero e modifica del dato
		virtual T& Data() = 0;

		// test fine lista
		virtual int EndOfList() const;
};

// Pone iterationComplete a 0 (False)
template <class T>
Iterator<T>::Iterator(): iterationComplete(0)
{}

// restituisce il valore iterationComplete.
template <class T>
int Iterator<T>::EndOfList() const
{
	return iterationComplete;
}


//CLASSE LIST

template <class T>
class List
{
	 protected:
		  // Numero di elementi della lista aggiornato dalle classi derivate
		  int size;
	 public:
		  // costruttore
		  List();

		  // distruttore virtuale necessario per classi derivate
		  // che usano allocazione di memoria dinamica
		  virtual ~List();

		  // metodi di accesso alla lista
		  virtual int ListSize() const;
		  virtual int ListEmpty() const;
		  virtual int Find (T& item) = 0;

		  // Metodi di modifica della lista: funzioni virtuali pure
		  virtual void Insert(const T& item) = 0;
		  virtual void Delete(const T& item) = 0;
		  virtual void ClearList() = 0;
};

// costruttore pone size a 0
template <class T>
List<T>::List(): size(0)
{}

//distruttore virtuale
template <class T>
List<T>::~List()
{}

// restituisce size
template <class T>
int List<T>::ListSize() const
{
	 return size;
}

// test per una lista vuota
template <class T>
int List<T>::ListEmpty() const
{
	 return size == 0;
}



//CLASSE NODE
template <class T>
class Node
{	Node<T> *next;	//next  l'indirizzo del nodo successivo
  public:
	T data;	//il dato  pubblico

	//Costruttore
	Node (const T& item, Node<T>* ptrnext =NULL);

	//Metodi per modificare la lista
	void InsertAfter (Node<T> *p);
	Node<T> *DeleteAfter ();

	//Metodo per avere l'indirizzo del nodo successivo
	Node<T> *NextNode () const;
};

//Implementazione della classe  Node

template <class T>
Node<T>::Node (const T& item, Node<T>* ptrnext) :
				data(item), next(ptrnext)
{}

template <class T>
void Node<T>::InsertAfter (Node<T> *p)
{  	//p punta al successore del nodo corrente
	//e il nodo corrente punta a p
	p->next =next;
	next =p;
}

template <class T>
Node<T> *Node<T>::DeleteAfter ()
{	//Salvare l'indirizzo del nodo da cancellare
	Node<T> *tempPtr =next;
	//Se non c' un successore restituisce NULL
	if (next == NULL)
		return NULL;
	//Il nodo corrente punta al successore di tempPtr
	next = tempPtr->next;
	//Restituisce il puntatore al nodo sconnesso
	return tempPtr;
}

template <class T>
Node<T> *Node<T>::NextNode () const
{	return next;
}


//Classe Lista linkata

template <class T>
class LinkedList
{	//Puntatori per accedere alla testa e alla coda della lista
	Node<T>  *front, *rear;
	//Puntatori per le operaz. di Insert e Delete
	Node<T> *prevPtr, *currPtr;
	//Numero di item nella lista
	int size;
	//Posizione nella lista , usata da Reset
	int position;
	//Metodi privati per allocare e deallocare nodi
	Node<T> *GetNode (const T& item, Node<T>* ptrNext=NULL);
	void FreeNode (Node<T> *p);
	//Copia lista L nella lista corrente
	void CopyList (const LinkedList<T>& L);
  public:
	//Costruttori
	LinkedList ();
	LinkedList (const LinkedList<T>& L);
	//Distruttore
	~ LinkedList ();

	//Operatore di assegnazione
	LinkedList<T>& operator= (const LinkedList<T>& L);

	//Metodi per verificare lo stato della lista
	int ListSize () const;
	int ListEmpty () const;

	//Metodi per attraversare la lista
	void Reset (int pos =0);
	void Next ();
	int EndOfList () const;
	int CurrentPosition () const;

	//Metodi per l'inserzione
	void InsertFront (const T& item);
	void InsertRear (const T& item);
	void InsertAt (const T& item);
	void InsertAfter (const T& item);

	//Metodi per cancellare
	T DeleteFront ();
	void DeleteAt ();

	//Modifica del contenuto informativo
	T& Data ();

	//Metodo per il clear della lista
	void ClearList ();

	//Metodo per dare elemento alla testa
	Node<T>* Front ();
};

 //Implementazione della lista LinkedList

//Metodi privati

template <class T>
Node<T> *LinkedList<T>::GetNode(const T& item,
							 Node<T>* ptrNext)
{
	Node<T> *p;

	p = new Node<T>(item,ptrNext);
	if (p == NULL)
	{
		cout << "Fallita allocazione memoria!\n";
		exit(1);
	}
	return p;
}

template <class T>
void LinkedList<T>::FreeNode(Node<T> *p)
{
	delete p;
}

// copia L nella lista corrente, che si assume vuota
template <class T>
void LinkedList<T>::CopyList(const LinkedList<T>& L)
{
	Node<T> *p = L.front;
	int pos;

	// inserisce ogni elemento in L in coda all'oggetto presente
	while (p != NULL)
	{
		InsertRear(p->data);
		p = p->NextNode();
	}

	// se lista  vuota return
	if (position == -1)
		return;

	// reset prevPtr e currPtr  nella nuova lista
	prevPtr = NULL;
	currPtr = front;
	for (pos = 0; pos != position; pos++)
	{
		prevPtr = currPtr;
		currPtr = currPtr->NextNode();
	}
}

//Metodi pubblici

template <class T>
LinkedList<T>::LinkedList ():
				front (NULL), rear (NULL), prevPtr (NULL),
				currPtr (NULL), size (0), position (-1)
{}

template <class T>
LinkedList<T>::LinkedList(const LinkedList<T>& L)
{
	front = rear = NULL;
	prevPtr = currPtr = NULL;
	size = 0;
	position = -1;
	CopyList(L);
}

template <class T>
void LinkedList<T>::ClearList()
{
	Node<T> *currPosition, *nextPosition;

	currPosition = front;
	while(currPosition != NULL)
	{
		// indirizzo del nodo successivo e cancella il nodo corrente
		nextPosition = currPosition->NextNode();
		FreeNode(currPosition);
		currPosition = nextPosition;
	}
	front = rear = NULL;
	prevPtr = currPtr = NULL;
	size = 0;
	position = -1;
}

template <class T>
LinkedList<T>::~LinkedList()
{
	ClearList();
}

template <class T>
LinkedList<T>& LinkedList<T>::operator=
					(const LinkedList<T>& L)
{
	if (this == &L)      //Non pu assegnare lista a s stessa
		return *this;

	ClearList();
	CopyList(L);
	return *this;
}

template <class T>
int LinkedList<T>::ListSize() const
{
	return size;
}

template <class T>
int LinkedList<T>::ListEmpty() const
{
	return size == 0;
}

// sposta avanti di un nodo prevPtr e currPtr
template <class T>
void LinkedList<T>::Next()
{
	// se la scansione ha raggiunto la fine della lista o
	// la lista  vuota, return
	if (currPtr != NULL)
	{
		// avanzano i due  puntatori al nodo successivo
		prevPtr = currPtr;
		currPtr = currPtr->NextNode();
		position++;
	}
}

// True se il cliente ha attraversato la lista
template <class T>
int LinkedList<T>::EndOfList() const
{
	return currPtr == NULL;
}

// restituisce la posizione del nodo corrente
template <class T>
int LinkedList<T>::CurrentPosition() const
{
	return position;
}

//Reset per attraversare la lista a partire dall'inizio
template <class T>
void LinkedList<T>::Reset (int pos)
{	int startPos;
	//se la lista  vuota return
	if (front == NULL)
		return;
	//se la posizione non  valida terminare il programma
	if (pos <0 || pos> size-1)
	{	cerr << "Reset: posizione non valida: " << pos << endl;
		return;
	}
	//Spostarsi sul nodo pos
	if (pos ==0)
	{	//reset alla testa lista
		prevPtr = NULL;
		currPtr = front;
	position =0;
	}
	else
	{	//Reset dei puntatori
		currPtr = front->NextNode ();
		prevPtr = front;
		startPos = 1;
		//Spostarsi fino a pos
		for (position =startPos; position != pos; position++)
		{	prevPtr = currPtr;
			currPtr = currPtr->NextNode ();
		}
	}
}

// restituisce un reference al valore nel nodo corrente
template <class T>
T& LinkedList<T>::Data()
{
	// errore se lista vota o scansione completa
	if (size == 0 || currPtr == NULL)
	{
		cerr << "Data: reference non valida!" << endl;
		exit(1);
	}
	return currPtr->data;
}

// Inserisce item alla testa di lista
template <class T>
void LinkedList<T>::InsertFront(const T& item)
{
	// chiama Reset se la lista non  vuota
	if (front != NULL)
		Reset();
	InsertAt(item);        // inserisce in testa
}

// Inserisce item in coda a lista
template <class T>
void LinkedList<T>::InsertRear(const T& item)
{
	Node<T> *newNode;

	prevPtr = rear;
	newNode = GetNode(item);	// crea il nuovo nodo
	if (rear == NULL)			// se lista vuota, inserisce in testa
		front = rear = newNode;
	else
	{
		rear->InsertAfter(newNode);
		rear = newNode;
	}
	currPtr = rear;
	position = size;
	size++;
}

// Inserisce item in posizione corrente
template <class T>
void LinkedList<T>::InsertAt(const T& item)
{
	Node<T> *newNode;

	// 2 casi: inserzione in testa o dentro la lista
	if (prevPtr == NULL)
	{
		// inserisce in testa,
		// pone nodo in una lista vuota
		newNode = GetNode(item,front);
		front = newNode;
	}
	else
	{
		// inserisce in lista, pone il nodo dopo prevPtr
		newNode = GetNode(item);
		prevPtr->InsertAfter(newNode);
	}

	// if prevPtr == rear, si sta inserendo in lista vuota
	// o in coda di una lista non vuota; aggiorna rear e position
	if (prevPtr == rear)
	{
		rear = newNode;
		position = size;
	}

	// aggiorna currPtr e incrementa size
	currPtr = newNode;
	size++;
}

// Inserisce item dopo la posizione corrente
template <class T>
void LinkedList<T>::InsertAfter(const T& item)
{
	Node<T> *p;

	p = GetNode(item);
	if (front == NULL)       // inserzione in lista vuota
	{
		front = currPtr = rear = p;
		position = 0;
	}
	else
	{
		// inserzione dopo l'ultimo nodo
		if (currPtr == NULL)
		  currPtr = prevPtr;
		currPtr->InsertAfter(p);
		if (currPtr == rear)
		{
		  rear = p;
		  position = size;
		}
		else
			position++;
			prevPtr = currPtr;
			currPtr = p;
	}
	size++;              // incrementa size
}

// Cancella nodo in testa
template <class T>
T LinkedList<T>::DeleteFront()
{
	T item;

	Reset();
	if (front == NULL)
	{
		cerr << "Cancellazione non valida!" << endl;
		exit(1);
	}
	item = currPtr->data;
	DeleteAt();
	return item;
}

//Cancella nodo in posizione corrente
template <class T>
void LinkedList<T>::DeleteAt()
{
	Node<T> *p;

	// error se lista vuota o fine lista
	if (currPtr == NULL)
	{
		cerr << "Cancellazione non valida!" << endl;
		exit(1);
	}

	// cancella zione in testa o dentro
	if (prevPtr == NULL)
	{
		// salva l'indirizzo della testa
		// se  ultimo nodo  front diventa NULL
		p = front;
		front = front->NextNode();
	}
	else
		// scollega nodo dopo prevPtr, salva indirizzo
		p = prevPtr->DeleteAfter();

	// se rear  cancellato, nuova rear  prevPtr e position
	//  decrementata; altrimenti, position  la stessa
	// se p era ultimo nodo, rear = NULL e position = -1
	if (p == rear)
	{
		rear = prevPtr;
		position--;
	}

	// sposta currPtr dopo il nodo cancellato, se p  ultimo nodo
	//currPtr diventa NULL
	currPtr = p->NextNode();

	// rilascia il nodo e  decrementa size
	cout<< "Cancellare in llist"<<endl;
	FreeNode(p);
	size--;
}

template <class T>
Node<T>* LinkedList<T>::Front ( )
{ return front;
}


// CLASSE SEQLIST

template <class T>
class SeqListIterator;

template <class T>
class SeqList: public List<T>
{
	protected:
		// oggetto lista linkata
		LinkedList<T> llist;

	public:
		// costruttore
		SeqList();

		// metodi di accesso alla lista
		virtual int Find (T& item);
		T GetData(int pos);

		// Metodi di modifica della lista
		virtual void Insert(const T& item);
		virtual void Delete(const T& item);
		T DeleteFront( );
		virtual void ClearList( );

		// SeqListIterator richiede di accedere a llist
		friend class SeqListIterator<T>;
};

// costruttore di default inizializza classe base
template <class T>
SeqList<T>::SeqList(): List<T>()
{}

// usa metodo ClearList per il clear di llist
template <class T>
void SeqList<T>::ClearList()
{
	llist.ClearList();
	size = 0;
}

// usa metodo InsertRear per aggiungere elementi in coda alla lista
template <class T>
void SeqList<T>::Insert(const T& item)
{
	llist.InsertRear(item);
	size++;	// aggiorna size
}

// usa metodo DeleteFront per rimuovere il I elemento
template <class T>
T SeqList<T>::DeleteFront()
{
	size--;
	return llist.DeleteFront();
}

// cancella nodo il cui valore  uguale ad item
template <class T>
void SeqList<T>::Delete(const T& item)
{
	int result = 0;

	// ricerca item se trova, pone result a true
	for(llist.Reset();!llist.EndOfList();llist.Next())
		if (item == llist.Data())
		{
			result++;
			break;
		}

	// se ha trovato item, lo cancella e decrementa size
	if (result)
	{  cout<<"cancellare"<<endl;
		llist.DeleteAt();
		size--;
	}
}

// restituisce il valore di item nella posizione pos
template <class T>
T SeqList<T>::GetData(int pos)
{ // verifica se  posizione valida
	if (pos < 0 || pos >= llist.ListSize())
	{
		cerr << "pos  fuori range!" << endl;
		exit(1);
	}
	// pone a pos posizione corrente nella linked list e return data
	llist.Reset(pos);
	return llist.Data();
}

// Ricerca item in lista, return True se trova
// False altrimenti. Se ha trovato
// assegna l'elemento al parametro reference item
template <class T>
int SeqList<T>::Find (T& item)
{
	int result = 0;

	// ricerca item in lista, se trova, set result a True
	for(llist.Reset();!llist.EndOfList();llist.Next())
		if (item == llist.Data())
		{
			result++;
			break;
		}

	// se True, aggiorna item e return True; else, return False
	if (result)
		item = llist.Data();
	return result;
}


//CLASSE SEQLISTITERATOR DERIVATA DA ITERATOR

template <class T>
class SeqListIterator:   public Iterator<T>
{
	private:
		// puntatore locale a SeqList che si sta scandendo
		SeqList<T> *listPtr;
		// mantiene puntatori previous e current
		// mentre si scandisce la lista
		Node<T> *prevPtr, *currPtr;

	public:
		// costruttore
		SeqListIterator(SeqList<T>& lst);

		// Metodi per la scansione
		virtual void Next();
		virtual void Reset();

		// Metodi per accedere e modificare il dato
		virtual T& Data();

		// fa il reset dell'iteratore per scandire una nuova lista
	  // l'equivalente del costruttore in run time
		void SetList(SeqList<T>& lst);
};

// costruttore: inizializza costruttore classe base e puntatore SeqList
template <class T>
SeqListIterator<T>::SeqListIterator(SeqList<T>& lst) :
		Iterator<T>(), listPtr(&lst)
{
	// La lista pu essere vuota
	iterationComplete = listPtr->llist.ListEmpty();
	// posiziona iterator al front della lista
	Reset();
}

// avanza al successivo elemento di lista
//quando arriva alla fine lista segnala che l'iterazione  completa
template <class T>
void SeqListIterator<T>::Next()
{
	//se currPtr  NULL, si  alla fine di lista
	if (currPtr == NULL)
		return;

	// si sposta prevPtr/currPtr avanti di un nodo
	prevPtr = currPtr;
	currPtr = currPtr->NextNode();

	// se si  alla fine della lista, segnala
	// che l'iterazione  completa
	if (currPtr == NULL)
		iterationComplete = 1;
}

// Inizializza la scansione all'inizio di  lista
template <class T>
void SeqListIterator<T>::Reset()
{
	// riasssegna lo stato della iteratione
	iterationComplete = listPtr->llist.ListEmpty();

	// se lista  vuota, return
	if (listPtr->llist.Front() == NULL)
		return;

	// inizializza al primo nodo della lista
	prevPtr = NULL;
	currPtr = listPtr->llist.Front();
}

// restituisce il valore del nodo corrente nella lista
template <class T>
T& SeqListIterator<T>::Data()
{
	// errore se lista  vuota o scansione completa
	if (listPtr->llist.ListEmpty() || currPtr == NULL)
	{
		cerr << "Data: riferimento non valido!" << endl;
		exit(1);
	}
	return currPtr->data;
}

//L'iteratore attraversa una nuova lista lst passata come parametro,
//riassegna listPtr e chiama Reset
template <class T>
void SeqListIterator<T>::SetList(SeqList<T>& lst)
{
	listPtr = &lst;

	// posiziona al primo dato nella nuova lista
	Reset();
}


/*******************************************************************************/
//Eugenio Conforti

//CLASSE ELEMENT

class Element
{
private:
	String Nome;
	char Tipo;
public:

//Costruttori
	Element();
	Element(String n, int cost);
	Element(Element& copy);

// Distruttore
	virtual~Element(){};

//Metodi di accesso
	String get_nome(){return Nome;}
	int get_Tipo(){return Tipo;}
	void set_nome(String n) {Nome=n;}
	void set_Tipo(int cost){Tipo=cost;}

//Metodi di stampa e inserimento
	virtual void display();
	virtual void input();

//Metodi virtuali delle classi derivate
	virtual float get_Saldo(){ return 0.0; }
	virtual float get_TotaA(){ return 0.0; }
	virtual int get_NConto(){ return 0; }
	virtual float get_TotP(){ return 0.0; }
	virtual void set_Saldo(float targ){}
	virtual void set_TotaA(float tip){}
	virtual void set_NConto(int tass){}
	virtual void set_TotP(float Tp){}
	virtual int get_Cod(){ return 0; }
	virtual float get_TotM(){ return 0.0; }
	virtual float get_TotA(){ return 0.0; }
	virtual void set_Cod(int a){}
	virtual void set_TotM(float a){}
	virtual void set_TotA(float a){}

	virtual void modifica(){}
};

//costruttore
Element::Element()
{
	Tipo=0;
}

//costruttore overloaded
Element::Element(String n, int cost)
{
	Nome=n;
	Tipo=cost;
}

//costruttore di copia
Element::Element(Element& copy)
{
	Nome=copy.get_nome();
	Tipo=copy.get_Tipo();
}

//metodo di stampa
void Element::display()
{
	cout<<"stampa dell'elemento: "<<endl;
	cout<<"Nome cliente: "<<Nome.GetString()<<endl;
	cout<<"Tipo di rapporto: ";
	switch(Tipo){
					case 1:cout<<"Conto corrente"<<endl;
							break;
					default:cout<<"Carta di credito"<<endl;
							break;
					}
}

//metodo di inserimento
void Element::input()
{
	cout<<"inserimento di un elemento: "<<endl;
	cout<<"\n Nome del cliente: "; Nome.Leggi();
}

//CLASSE CONTO DERIVATA DI ELEMENT

class Conto:public Element
{
private:
	float Saldo;
	float TotaA;
	int NConto;
	float TotP;

public:

	//costruttori e distruttore
	Conto();
	Conto(float Sa, float  Ta, int Nc, float Tp);
	Conto(Conto&);
	~Conto(){}

	//metodi di accesso
	virtual float get_Saldo(){return Saldo;}
	virtual float get_TotaA(){return TotaA;}
	virtual int get_NConto(){return NConto;}
	virtual float get_TotP(){return TotP;}
	virtual void set_Saldo(float targ){Saldo=targ;}
	virtual void set_TotaA(float tip){TotaA=tip;}
	virtual void set_NConto(int tass){NConto=tass;}
	virtual void set_TotP(float Tp){TotP=Tp;}

	//metodo di stampa
	virtual void display();
	//metodo di inserimento
	virtual void input();
	virtual void modifica();
};

//costruttore
Conto::Conto()
{
	Saldo=0;
	TotaA=0;
	NConto=0;
	TotP=0;
}

//costruttore overloded
Conto::Conto(float Sa, float Ta, int Nc, float Tp)
{
	Saldo=Sa;
	TotaA=Ta;
	NConto=Nc;
	TotP=Tp;
}

//Costruttore di copia
Conto::Conto(Conto& copy)
{
	Saldo=copy.get_Saldo();
	TotaA=copy.get_TotaA();
	NConto=copy.get_NConto();
	TotP=copy.get_TotP();
}

//Metodo di stampa della classe
void Conto::display()
{
	Element::display();
	cout<<"Numero di conto: "<<NConto<<endl;
	cout<<"Saldo del conto: "<<Saldo<<endl;
	cout<<"Cifra totale degli addebiti: "<<TotaA<<endl;
	cout<<"Cifra totale dei prelievi: "<<TotP<<endl;
}

//Metodo di inserimento per la classe
void Conto::input()
{
	Element::input();
	cout<<"inserire il numero di conto"<<endl;
	cin>>NConto;
	cout<<"inserire il Saldo del conto"<<endl;
	cin>>Saldo;
	cout<<"inserire cifra totale degli addebiti"<<endl;
	cin>>TotaA;
	cout<<"inserire cifra totale dei prelievi"<<endl;
	cin>>TotP;
}

//Metodo per la modifica
void Conto::modifica()
{
	int a;
	cout<<"Modifica del Conto(1=addebito, 2=prelievi): "<<endl;
	cin>>a;
	float mod;
	switch(a){
				case 1:cout<<"\n inserire l'addebito: ";
						 cin>>mod;
						 TotaA+=mod;
						 Saldo+=mod;
						 break;
				case 2:cout<<"\n inserire il prelievo: ";
						 cin>>mod;
						 TotP+=mod;
						 Saldo=Saldo-mod;
						 break;
				}
}

//CLASSE Carta DERIVATA DA ELEMENT

class Carta:public Element
{
private:
	int Cod;
	float TotM;
	float TotA;
public:

	//costruttori e distruttore
	Carta();
	Carta(int,float,float);
	Carta(Carta&);
	~Carta(){}

	//metodi di accesso
	virtual int get_Cod(){return Cod;}
	virtual float get_TotM(){return TotM;}
	virtual float get_TotA(){return TotA;}
	virtual void set_Cod(int a){Cod=a;}
	virtual void set_TotM(float a){TotM=a;}
	virtual void set_TotA(float a){TotA=a;}

	//metodo di stampa
	virtual void display();
	//metodo di inserimento
	virtual void input();
	//metodo di modifica dei dati
	virtual void modifica();
};

//costruttore
Carta::Carta()
{
	Cod=0;
	TotM=0;
	TotA=0;
}

//costruttore overloaded
Carta::Carta(int a,float m,float p)
{
	Cod=a;
	TotM=m;
	TotA=p;
}

//costruttore di copia
Carta::Carta(Carta& copy)
{
	Cod=copy.get_Cod();
	TotM=copy.get_TotM();
	TotA=copy.get_TotA();
}

//Metodo di stampa
void Carta::display()
{
	Element::display();
	cout<<"Numero di carta di credito: "<<Cod<<endl;
	cout<<"Spese totali dell'ultimo mese: "<<TotM<<endl;
	cout<<"Spese totali dell'ultimo anno: "<<TotA<<endl;
}

//metodo di stampa
void Carta::input()
{
	Element::input();
	cout<<"\n inserire il numero di carta di credito ";
	cin>>Cod;
	cout<<"\n inserire le spese totali dell'ultimo mes: ";
	cin>>TotM;
	cout<<"\n inserire le spese totali dell'ultimo anno: ";
	cin>>TotA;
}

//metodo di modifica
void Carta::modifica()
{
	cout<<" aquisto con carta, inserire la spesa: "<<endl;
	float mod;
	cin>>mod;
	TotM+=mod;
	TotA+=mod;
}
//Classe ARCHIVIO PER L'INSERIMENTO DEGLI ELEMENTI
class Archivio
{
private:
			SeqList<Element *> Arch;
			SeqListIterator<Element *> Iter;
public:
			//costruttore e distruttore
			Archivio():Arch(),Iter(Arch){}
			~Archivio();

			//metodi di accesso
			SeqList<Element *> GetArch()const{return Arch;}

			// Elaborazioni richieste
			void inserisci();
			void modifica();
			void stampa();
			void indagine();

};

//distruttore
Archivio::~Archivio()
{
	for (Iter.Reset();!Iter.EndOfList();Iter.Next())
	delete Iter.Data();
}

//metodo di inserimento
void Archivio::inserisci()
{  Element *p;
	char choice='?';
	cout<<"          1 - Conto corrente"<<endl;
	cout<<"          2 - Carta di credito"<<endl;
	cout<<"SCELTA: ";
	cin>>choice;
	switch(choice)
		 {case '1' :p=new Conto;
						break;
		  case '2' :p=new Carta;
						break;
		  default:cout<<"SCELTA ERRATA"<<endl;
						return;
		 }
	p->set_Tipo(choice);
	p->input();
	Arch.Insert(p);
	cout<<"                    *Dato inserito*\n";
}

//metodo di modifica
void Archivio::modifica()
{
	String Nom;
	cout<<"\n Inserire il nome del cliente che intende fare operazioni di modifica: "<<endl;
	Nom.Leggi();
	for (Iter.Reset(); !Iter.EndOfList(); Iter.Next())
		{
		if (Iter.Data()->get_nome()==Nom)
			break;
		}
	 Iter.Data()->modifica();
}

//metodo di stampa
 void Archivio::stampa()
 { cout<<"\n      ARCH:\n";
	for(Iter.Reset();!Iter.EndOfList();Iter.Next())
	{ Iter.Data()->display();
	  cout<<"---------------------------------------------------------------------\n";
	  cout<<"             *PREMERE UN TASTO PER CONTINUARE*\n";
	  getch();
	}
 }

//metodo di indagine statistica
void Archivio::indagine()
{
	int n=0;
	float add=0;
	float prel=0;
	for(Iter.Reset();!Iter.EndOfList();Iter.Next())
		{
			if ((Iter.Data()->get_Saldo()>=30000000.0) && (Iter.Data()->get_Saldo()<=60000000.0))
				{
					n++;
					add+=Iter.Data()->get_TotaA();
					prel+=Iter.Data()->get_TotP();
				 }
		}
	if (n!=0)
	  {
		add=add/n;
		prel=prel/n;
		cout<<"Il numero di clienti con conto compreso tra 30 e 60 milioni : "<<n<<endl;
		cout<<"La loro media di addebito : "<<add<<endl;
		cout<<"La loro media di prelievo : "<<prel<<endl;
		}
	else
		cout<<"Non si puo' calcolare la media"<< endl;
}


//MAIN
void main()
 { Archivio A;
	char choice='?';
	while(choice!='u')
		  { cout<<"\n   ***************************MENU***************************"<<endl;
			 cout<<"          1 - Inserimento"<<endl;
			 cout<<"          2 - Stampa Arch"<<endl;
			 cout<<"          3 - operazioni di modifica"<<endl;
			 cout<<"          4 - indagine statistica sui conto"<<endl;
			 cout<<"          u - Per uscire"<<endl;
			 cout<<"   **********************************************************"<<endl;
			 cout<<"SCELTA: ";
			 cin>>choice;
			 switch(choice)
					{ case '1' :A.inserisci();
									break;
					  case '2' :A.stampa();
									break;
					  case '3' :A.modifica();
									break;
					  case '4' :A.indagine();
									break;
					  case 'u' :cout<<"Fine programma";
									break;
					  default:cout<<"SCELTA ERRATA"<<endl;
					}
		  }
 }








